<?php

namespace Drupal\cforge\Forms;

use Drupal\mcapi\Entity\Wallet;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Form builder for Cforge settings.
 */
class Settings extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'cforge_settings';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildform($form, $form_state);
    $config = $this->config('cforge.settings');

    $committee = \Drupal::entityTypeManager()
      ->getStorage('user')
      ->loadByProperties(['roles' => 'committee']);
    $form['account_manager'] = [
      '#title' => $this->t('Account manager'),
      '#description' => $committee ? $this->t('The committee member who handles new members.') : 'No committee members',
      '#type' => 'select',
      '#options' => array_map(function ($c) {return $c->getDisplayName();}, $committee),
      '#default_value' => \Drupal::Config('cforge.settings')->get('account_manager'),
      '#required' => FALSE,
      '#weight' => 1
    ];

    $terminology = [];
    foreach ($config->get('terminology') as $key => $value) {
      $terminology[] = "$key|$value";
    }
    $form['terminology'] = [
      '#title' => $this->t('Replace words'),
      '#description' => $this->t('This is applied to all interface strings after translation (i.e. not config nor content) Case sensitive. No dots allowed.'),
      '#type' => 'textarea',
      '#default_value' => implode("\n", $terminology),
      '#element_validate' => [[get_class(), 'validateTerminology']],
      '#rows' => 8,
      '#weight' => 4
    ];
    $form['robots_protect'] = [
      '#title' => $this->t('Conceal from search engines'),
      '#description' => $this->t('Rewrites the robots.txt file'),
      '#type' => 'checkbox',
      '#default_value' => $config->get('robots_protect', 0),
      '#weight' => 5,
    ];
    $form['mail_footer'] = [
      '#title' => $this->t('Footer'),
      '#description' => $this->t('Appended to all mails sent by the site to members. HTML is allowed.'),
      '#type' => 'textarea',
      '#rows' => 4,
      '#default_value' => $config->get('mail_footer'),
      '#weight' => 6,
    ];

    return $form;
  }

  /**
   * Element validation callback.
   */
  public static function validateTerminology(&$element, $form_state) {
    try {
      $array = static::extractAllowedValues($element['#value']);
    }
    catch (\Exception $e){
      $form_state->setError($element, $e->getMessage());
    }
    $form_state->setValue('terminology', $array);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $values = $form_state->getValues();
    $config = $this->configFactory->getEditable('cforge.settings');
    $config
      ->set('mail_footer', $values['mail_footer'])
      ->set('robots_protect', $values['robots_protect'])
      ->set('terminology', $form_state->getValue('terminology'))
      ->set('account_manager', $form_state->getValue('account_manager'))
      ->save();
  }

  /**
   * {@inheritdoc}
   */
  public function getEditableConfigNames() {
    return ['cforge.settings'];
  }

  /**
   * Extracts the allowed values array from the terminology element.
   *
   * @param string $string
   *   The raw string to extract values from.
   *
   * @return array
   *   The array of extracted key/value pairs, or NULL if the string is invalid.
   *
   * @throws Exception
   */
  protected static function extractAllowedValues($string) {
    $values = [];
    $list = explode("\n", $string);
    $list = array_map('trim', $list);
    $list = array_filter($list, 'strlen');
    $generated_keys = FALSE;
    foreach ($list as $text) {
      $matches = [];
      if (preg_match('/(.+)+\|(.+)/', $text, $matches)) {
        // Trim key and value to avoid unwanted spaces issues.
        $key = trim($matches[1]);
        $value = trim($matches[2]);
      }
      else {
        throw new \Exception(t("Could not parse terminology key/value pair: $text"));
      }
      $values[$key] = $value;
    }
    return $values;
  }

}
